#!/usr/bin/env python

import sys

def move_editions(tdiff, sign, source):
	while source:
		rec = sign + source.pop(0)
		tdiff.append(rec)

def prune_equal_header(t1, t2):
	tdiff = []
	while t1 and t2 and t1[0] == t2[0]:
		tdiff.append(" " + t1[0])
		t1.pop(0)
		t2.pop(0)
	return tdiff

def prune_equal_tail(t1, t2):
	tdiff_tail = []
	while t1 and t2 and t1[-1] == t2[-1]:
		tdiff_tail.insert(0, " " + t1[-1])
		t1.pop()
		t2.pop()
	return tdiff_tail

def find_lcstr_recursive(t1, t2):
	# Remove trivial parts (common heads and tails)
	head = prune_equal_header(t1, t2)
	middle = []
	tail = prune_equal_tail(t1, t2)

	# Find longest common string (LCS)
	longest = 0
	longest_t1_start = 0
	longest_t2_start = 0

	for i in range(0, len(t1)):
		length = 0
		t1_start = i
		t2_start = 0
		for j in range(0, min(len(t2), len(t1) - i)):
			if t1[i + j] == t2[j]:
				if length == 0:
					t2_start = j
					t1_start = i + j
				length += 1
			elif length > 0:
				# end of common substring 
				break

		if length > longest:
			longest = length
			longest_t1_start = t1_start
			longest_t2_start = t2_start

	if not longest:
		# No common substring
		move_editions(middle, "-", t1)
		move_editions(middle, "+", t2)
	else:
		print "Longest: " + "".join(t1[longest_t1_start:longest_t1_start + longest])
		# Proceed recursively before and after LCS
		middle.extend(find_lcstr_recursive(t1[:longest_t1_start], t2[:longest_t2_start]))
		move_editions(middle, " ", t1[longest_t1_start:longest_t1_start + longest])
		middle.extend(find_lcstr_recursive(t1[longest_t1_start + longest:], \
						 t2[longest_t2_start + longest:]))

	tdiff = []
	tdiff.extend(head)
	tdiff.extend(middle)
	tdiff.extend(tail)
	return tdiff

def cinediff(t1, t2):
	tdiff = find_lcstr_recursive(t1, t2)
	return tdiff

t1 = [ x for x in sys.argv[1] ]
t2 = [ x for x in sys.argv[2] ]

tdiff = cinediff(t1, t2)

for l in tdiff:
	print l,
