#!/usr/bin/env python

import sys

def move_editions(tdiff, sign, source):
	while source:
		rec = sign + source.pop(0)
		tdiff.append(rec)

def find_k_candidates(t1, t2):
	kc = []
	k = 0
	# dummy 0-candidate
	K = [[[0,0,0, None]]]
	fence_j1 = [ 0 for _ in range(0, len(t1) + 1) ]
	todo = True

	# Loop for progressively higher k-candidates
	for k in range(1, 1 + min(len(t1), len(t2))):
		# print "k:", k
		kc = []
		K.append(kc)
		# left fence: leftmost (k-1)-candidate + 1
		fence_i = K[-2][0][0] + 1
		# lower fence is irregular and was planted by (k-1)-candidates
		fence_j0 = fence_j1
		# higher fence is, for now, the graph border
		fence_j1 = [ len(t2) for _ in range(0, len(t1) + 1) ]
		for i in range(fence_i, len(t1)):
			# print "\ti:", i
			# print "\t\t i-fence: %d j-fence: %d-%d" % (fence_i, fence_j0[i] + 1, fence_j1[i])
			for j in range(fence_j0[i] + 1, fence_j1[i]):
				# print "\t\tj:", j
				if t1[i] == t2[j]:
					# print "\t\t\tFound"
					# k-candidate: plant a fence here
					# next k-candidates can't share this row
					for ii in range(i + 1, len(fence_j1)):
						fence_j1[ii] = j
					kc.append([i, j, k, t1[i]])
					# no two k-candidates can share a column
					break
		if not kc:
			# dummy ending k-candidate
			kc.append([len(t1), len(t2), k, None])
			break

	# print "K-candidates:"
	# for k in range(0, len(K)):
	#	print K[k]
	# print "--------------"


	return K

# Find a good path (not necessarily the shortest)
def find_path(K):
	K = K[:]
	# Add dummy beginning
	path = [K[0][0]]
	k = 1
	while k < (len(K) - 1):
		# print K[k], "next", K[k+1]
		if K[k][-1][2] != k:
			raise "Internal error"
		candidate = K[k][-1]
		# print "Candidate:", candidate
		if candidate[0] < path[-1][0] or candidate[1] < path[-1][1]:
			# it is behind the limit set by previous cycle
			# retreat and find another route
			del path[-1]
			k -= 1
			continue

		# is there a suitable k+1 candidate?
		for next_candidate in reversed(K[k+1]):
			if next_candidate[0] > candidate[0] and next_candidate[1] > candidate[1]:
				# yes, there is: annotate and go on
				path.append(candidate)
				k += 1
				break
		else:
			# no, there is not. This is a dead end
			if len(K[k]) == 1:
				raise "Internal error"
			del K[k][-1]

	# Add dummy end
	path.append(K[-1][-1])
	return path

# Find shortest path (uses more memory)
def find_shortest_path(K, t1, t2):
	K = K[:]
	paths = []
	costs = []

	paths.append([K[0][0]])
	costs.append(0)
	for k in range(1, len(K)):
		old_paths = paths
		old_costs = costs
		paths = []
		costs = []
		# create all possible paths, from current paths to current k-candidates
		for i in range(0, len(old_paths)):
	 		for j in range(0, len(K[k])):
				kcand = K[k][j]
				path = old_paths[i][:]
				if kcand[0] < path[-1][0] or kcand[1] < path[-1][1]:
					# invalid path extension (retreating is forbidden)
					continue
				cost = old_costs[i]
				cost += (kcand[0] - path[-1][0] - 1) + (kcand[1] - path[-1][1] - 1)
				path.append(kcand)
				paths.append(path)
				costs.append(cost)

		# prune paths that lead to the same given k-candidate but are longer or redundant
		for j in range(0, len(K[k])):
			# Select a k-candidate
			path_x = K[k][j][0]
			path_y = K[k][j][1]
			# Check all paths that lead to this k-candidate,
			# find the lowest cost among them
			i = 0
			cost = len(t1) + len(t2) + 1
			for i in range(0, len(paths)):
				if paths[i][-1][0] != path_x or paths[i][-1][1] != path_y:
					continue
				cost = min(costs[i], cost)
			# Remove all paths that have higher cost or are redundant
			i = 0
			while i < len(paths):
				if paths[i][-1][0] != path_x or paths[i][-1][1] != path_y:
					i += 1
				elif costs[i] > cost:
					del costs[i]
					del paths[i]
				elif costs[i] == cost:
					# Remove even equal-cost paths from now on
					cost -= 1
					i += 1
				else:
					i += 1
			
	return paths[0]

def generate_editions(path, t1, t2):
	tdiff = []
	last_i = 0
	last_j = 0
	for i, j, k, s in path:
		move_editions(tdiff, "-", t1[last_i + 1 : i])
		move_editions(tdiff, "+", t2[last_j + 1 : j])
		if s is not None:
			# i.e. k-candidate is not a dummy
			move_editions(tdiff, " ", t1[i:i+1])
		last_i = i
		last_j = j
	return tdiff

def diff_alt(t1, t2):
	K = find_k_candidates(t1, t2)
	path_alt = find_shortest_path(K, t1, t2)
	return generate_editions(path_alt, t1, t2)

def diff(t1, t2):
	K = find_k_candidates(t1, t2)
	path = find_path(K)
	return generate_editions(path, t1, t2)

t1 = [ x for x in sys.argv[1] ]
t1.insert(0, "$")
t2 = [ x for x in sys.argv[2] ]
t2.insert(0, "$")

tdiff_alt = diff_alt(t1, t2)
for l in tdiff_alt:
	print l,
print "    Optimal edition"

tdiff = diff(t1, t2)
for l in tdiff:
	print l,
print "  Sub-optimal edition"
